<?php
if ( !class_exists( 'WP_Mailer_Logger' ) )
{
/*
	Plugin Name: WP Mailer Logger
	Description: Appends some logging to mail content using the new filters of wp_mailer class
	Author: arena
	Version: 0
*/
class WP_Mailer_Logger
{
        const route_to_dpo = array('user_request_confirmed_email');

	function __construct() 
	{
		add_filter( 'wp_mailer_mail', array( __CLASS__, 'append_log' ), 999, 2 );
	}

	public static function append_log( $wp_mail, $call ) 
	{
		$len = 20;
		$len2 = 150;
		$sep = '+-' . self::str_format( '', $len, '-' )  . '-+-' . self::str_format( '', $len2, '-' ) . '-+';

                $tot = strlen( $sep );
                $str = ' ** ' . __CLASS__ . ' ** ';
                $repeat = floor( $tot / strlen($str) );
                $reste = $tot - ($repeat * strlen($str));
                if ( $reste > 1 ) {
                    $x = str_repeat( ' ', floor($reste/2)) . str_repeat( $str, $repeat);
                }
                else {
                    $x = str_repeat( $str, $repeat);
                }

		$log = array();

		/* head */
       		$log[] = '';
       		$log[] = '';
       		$log[] = '';
                $log[] = self::str_format( '', $tot, '=' );    
  		$log[] = $x;
                $log[] = self::str_format( '', $tot, '=' );
		$log[] = '          {{ Doubled curled brackets {{x}} are logged with doubled parenthesis ((x)) to avoid replacements }}';
		$log[] = $sep;

		/* body */
		/* caller */
		foreach( array( 'file', 'class', 'function', 'line', ) as $v )
		{
			if ( isset( $call[$v] ) )
			{
				$log[] = '! ' . self::str_format( $v, $len ) . ' ! ' .  self::str_format( $call[$v], $len2 ) . ' !';
			}
		}
		if ( isset( $call['args'] ) )
		{
			foreach( $call['args'] as $k => $v )
			{
				$log[] = '! ' . self::str_format( '-- arg[' . $k . ']', $len ) . ' ! ' .  self::type_format( $v, $len2 ) . ' !';
			}
		}

		/* $wp_mail */
		$log[] = $sep;
		foreach( array( 'id', 'group', 'subject', 'headers', ) as $v )
		{
			if ( isset( $wp_mail[$v] ) )
			{
				$log[] = '! ' . self::str_format( $v, $len ) . ' ! ' .  self::type_format( $wp_mail[$v], $len2 ) . ' !';
			}
		}

		$log[] = $sep;

		foreach( $wp_mail['replacements'] as $k => $v )
		{
			$log[] = '! ' . self::str_format( $k, $len ) . ' ! ' .  self::str_format( $v, $len2 ) . ' !';
		}

		/* footer */
		$log[] = $sep;

		/* append to mail footer */
		$wp_mail['replacements']['{{FOOTER}}'] .= implode( "\r\n", $log );

                return $wp_mail;
	}

	public static function type_format( $s, $l, $c = ' ' ) {
		switch( gettype( $s ) )
		{
			case 'boolean' :
			case 'integer' :
			case 'double' :
			case 'string' :
				$s = (string) $s;
			break;
			case 'array' :
				$s = '(array) > ' . json_encode( $s );
			break;
			case 'object' :
				$obj = get_class( $s );
				$s = '(object) ' . $obj . ' > ' . json_encode( $s );
			break;
			case 'resource' :
				$s = '(resource) ' . get_resource_type( $s );
			break;
			case 'NULL' :
				$s = 'NULL';
			break;
			default :
				$s = '(unknown type)';
			break;
		}
		return self::str_format( $s, $l, $c );
        }

	public static function str_format( $s, $l, $c = ' ' ) {
		$s = str_replace(array("\n", "\r", '  ', '  ', '  ', '{{', '}}'), array(' ', ' ', ' ', ' ', ' ', '((', '))') , $s);
		$s = trim( $s );
		$s = ( strlen( $s ) > $l ) ? substr( $s, 0, $l - 2 ) . ' +' : $s . str_repeat( $c, $l - strlen( $s ) );
		return $s;
        }	
}
new WP_Mailer_Logger();
}